@@ -20,6 +20,7 @@ import socket
 import sys
 #import threading
 #import time
+import urllib
 #import uuid
 
 # third-party module(s)
@@ -95,13 +96,23 @@ def getStartedInstances( db ):
     for collName in collNames:
         #logger.info( 'getting instances from %s', collName )
         launchedColl = db[collName]
-        inRecs = list( launchedColl.find() ) # fully iterates the cursor, getting all records
+        inRecs = list( launchedColl.find( {}, 
+            {'device-id': 1, 'cpu': 1, 'instanceId': 1, 'state': 1 }) 
+            ) # fully iterates the cursor, getting all records
         if len(inRecs) <= 0:
             logger.warn( 'no launched instances found in %s', collName )
         for inRec in inRecs:
             if 'instanceId' not in inRec:
                 logger.warning( 'no instance ID in input record')
-            inRec['dpr'] = round( instanceDpr( inRec ) )
+            if 'dpr' in inRec:
+                dpr = inRec['dpr']
+                if dpr < 24:
+                    logger.info( 'low dpr %.1f %s', dpr, inRec )
+            else:
+                dpr = instanceDpr( inRec )
+                if dpr < 24:
+                    logger.info( 'low dpr computed %.1f %s', dpr, inRec )
+            inRec['dpr'] = round( dpr )
         startedInstances.extend( [inst for inst in inRecs if inst['state'] in ['started', 'stopped'] ] )
     return startedInstances
 
@@ -255,7 +266,11 @@ def maybeNumber( txt ):
 def retrieveAccountInfo( projUrl, authStr ):
     reqUrl = projUrl + 'show_user.php?format=xml&auth=' + authStr
     resp = requests.get( reqUrl )  # do need this
-    tree = lxml.etree.fromstring(resp.content)
+    try:
+        tree = lxml.etree.fromstring(resp.content)
+    except Exception as exc:
+        logger.error( 'exception from lxml (%s) %s', type(exc), exc )
+        return {}
     
     hostInfos = []
     accountInfo = {}
@@ -288,6 +303,7 @@ if __name__ == "__main__":
         fromfile_prefix_chars='@', formatter_class=argparse.ArgumentDefaultsHelpFormatter )
     ap.add_argument( '--authToken', help='the NCS authorization token to use (required for launch or terminate)' )
     ap.add_argument( '--farm', required=True, help='the name of the virtual boinc farm' )
+    ap.add_argument( '--projectUrl', help='the URL to the boinc science project', default='https://boinc.bakerlab.org/rosetta/' )
     ap.add_argument( '--projectKey', required=True, help='the authorization key for the boinc project')
     ap.add_argument( '--mongoHost', help='the host of mongodb server', default='localhost')
     ap.add_argument( '--mongoPort', help='the port of mongodb server', default=27017)
@@ -295,12 +311,12 @@ if __name__ == "__main__":
 
     farm = args.farm  # 'rosetta_2'
     
-    logger.info( 'connecting to database' )
+    logger.info( 'connecting to database for %s', farm )
     mclient = pymongo.MongoClient( args.mongoHost )
     dbName = 'boinc_' + farm
     db = mclient[dbName]
 
-    lookbackMinutes = 90
+    lookbackMinutes = 120
     thresholdDateTime = datetime.datetime.now( datetime.timezone.utc ) \
         - datetime.timedelta( minutes=lookbackMinutes )
     thresholdDateTimeStr = thresholdDateTime.isoformat()
@@ -313,13 +329,16 @@ if __name__ == "__main__":
     instancesByIid = {inst['instanceId']: inst for inst in startedInstances }
 
     # rosetta
-    projUrl = 'https://boinc.bakerlab.org/rosetta/'
+    projUrl = args.projectUrl  # 'https://boinc.bakerlab.org/rosetta/'
     authStr = args.projectKey  # account key for boinc project ("weak" key may not work)
     
     logger.info( 'retrieving account info from %s', projUrl )
     accountInfo = retrieveAccountInfo( projUrl, authStr )
-    hostInfos = accountInfo['hosts']
-    hostInfoByName = { host['domain_name']: host for host in hostInfos }
+    hostInfos = accountInfo.get('hosts')
+    if hostInfos:
+        hostInfoByName = { host['domain_name']: host for host in hostInfos }
+    else:
+        hostInfoByName = {}
 
 
     if not True:
@@ -376,11 +395,14 @@ if __name__ == "__main__":
             inst['unplugged'] = unplugged
         instHost = inst['ssh']['host']
         inst['hostName'] = instHost
-        if 'ztka' in instHost:
-            logger.warning( 'DOWNLOADERR %s', inst['_id'] )
+        #if 'ztka' in instHost:
+        #    logger.warning( 'DOWNLOADERR %s', inst['_id'] )
         if instHost and instHost in hostInfoByName:
             inst['totCredit'] = hostInfoByName[instHost]['total_credit']
             inst['RAC'] = hostInfoByName[instHost]['expavg_credit']
+        else:
+            inst['totCredit'] = 0
+            inst['RAC'] = 0
             
     # this loop prints info for recently-checked terminated instances
     print( 'recently terminated instances for farm', farm )
@@ -416,7 +438,9 @@ if __name__ == "__main__":
     print()
     print(datetime.datetime.now( datetime.timezone.utc ))
     projName = projUrl.split('/')[-2]
-    print( 'project', projName, 'stats' )
+    projNetLoc = urllib.parse.urlparse( projUrl ).netloc
+    #print( 'project', projNetLoc, 'stats' )
+    print( 'stats for project %s (%s)' % (projName, projNetLoc)  )
     if 'total_credit' not in accountInfo:
         print( 'not available' )
         print( accountInfo.get( 'error_msg', '' ) )
@@ -433,10 +457,14 @@ if __name__ == "__main__":
           % (nSuccessfulInstances, nCheckedInstances) )
     print()
     # print details for the best instances with finished tasks
+    if len( checkedInstancesDf ) > 20:
+        bestCreditThresh = max( sorted(checkedInstancesDf.totCredit)[-20], 1 )
+    else:
+        bestCreditThresh = 1
     print( 'best instances' )
     for inst in checkedInstances:
         #if inst.get('nTasksComputed',0) > 0:
-        if inst.get('totCredit',0) >= 7000:
+        if inst.get('totCredit',0) >= bestCreditThresh:  # 12000 150 300
             cdt = inst['checkedDateTime']
             cdtAbbrev = cdt.strftime( dateTimeTagFormat )[5:-2]
             iid = inst['_id']
@@ -479,7 +507,9 @@ if __name__ == "__main__":
     #ramByDevId = {}
     #for inst in startedInstances:
     #    ramByDevId[ inst['device-id'] ] = inst['ram']['total'] / 1000000
-    lowRiders = checkedInstancesDf[(checkedInstancesDf.dpr<39) & (checkedInstancesDf.dpr>=24)]
+    lowRiders = checkedInstancesDf[(checkedInstancesDf.dpr<38) & (checkedInstancesDf.dpr>=24)]
+    if 'totCredit' in lowRiders:
+        print( 'silver (sub-38) total credit:', round(lowRiders.totCredit.sum()))
     
     if False:
         reportExceptionRecoveriesHackerly( db )
@@ -504,5 +534,3 @@ if __name__ == "__main__":
                     for event in instEvents:
                         if event['category'] == 'charger':
                             logger.info( '%s %s', abbrevIid, event )
-    if 'totCredit' in lowRiders:
-        print( 'sub-33 total credit:', round(lowRiders[lowRiders.dpr<33].totCredit.sum()))
